# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_experiment', '0009_auto_20170804_0850'),
    ]

    operations = [
        migrations.CreateModel(
            name='CaseBelongingness',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
        ),
        migrations.CreateModel(
            name='CaseKeyValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('case', models.ForeignKey(to='dj_experiment.Case')),
                ('value', models.ForeignKey(to='dj_experiment.Value')),
            ],
        ),
        migrations.CreateModel(
            name='Dataset',
            fields=[
                ('basemodel_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='dj_experiment.BaseModel')),
                ('dsfilename', models.CharField(unique=True, max_length=256, db_index=True)),
                ('dsfile', models.FilePathField(path='./', max_length=1024, recursive=True)),
                ('casekeyvalues', models.ManyToManyField(to='dj_experiment.CaseKeyValue', through='dj_experiment.CaseBelongingness')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
            bases=('dj_experiment.basemodel',),
        ),
        migrations.CreateModel(
            name='FieldBelongingness',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('dataset', models.ForeignKey(to='dj_experiment.Dataset')),
            ],
        ),
        migrations.CreateModel(
            name='FieldKeyValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('fieldname', models.ForeignKey(to='dj_experiment.FieldGroup')),
                ('value', models.ForeignKey(to='dj_experiment.Value')),
            ],
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='name',
            field=models.CharField(unique=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default='./', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default='.', max_length=1),
        ),
        migrations.AddField(
            model_name='fieldbelongingness',
            name='keyvalue',
            field=models.ForeignKey(to='dj_experiment.FieldKeyValue'),
        ),
        migrations.AddField(
            model_name='dataset',
            name='fieldkeyvalues',
            field=models.ManyToManyField(to='dj_experiment.FieldKeyValue', through='dj_experiment.FieldBelongingness'),
        ),
        migrations.AddField(
            model_name='casebelongingness',
            name='dataset',
            field=models.ForeignKey(to='dj_experiment.Dataset'),
        ),
        migrations.AddField(
            model_name='casebelongingness',
            name='keyvalue',
            field=models.ForeignKey(to='dj_experiment.CaseKeyValue'),
        ),
    ]



import logging

from dj_experiment.models import Catalog
from django.core.management.base import BaseCommand, CommandError
from humanfriendly.tables import format_smart_table


class Command(BaseCommand):
    """Retrieve the list of experiments from a catalog."""

    help = 'Retrieve the list of experiments from a catalog'

    def add_arguments(self, parser):
        """Add arguments for handling the command."""
        # Named (optional) arguments start with --
        parser.add_argument('-c', '--catalog',  # argument flag
                            dest='catalog',  # argument name
                            default=None,
                            help='Query catalog by its name',
                            )

    def handle(self, *args, **options):
        """List all the entries from a catalog name."""
        if not options.get('catalog'):
            raise CommandError(
                "listexperimentsbycatalog wants at least one named argument"
            )
        else:
            catalog_name = options['catalog']
        column_names = ['Experiment Name']

        experiments = (
            cat.xperiments.values('name')
            for cat in Catalog.objects.filter(
                name=catalog_name
            )
        )
        explist = list()
        for xperiment in experiments:
            for exp_kws in xperiment:
                explist.append([exp_kws['name'], ])

        self.stdout.write(format_smart_table(
            explist, column_names))

        logging.info('Command executed successfully!')



import logging
import os

from dj_experiment.models import Case as CaseModel
from dj_experiment.models import Experiment as ExperimentModel
from dj_experiment.models import FieldGroup as FieldGroupModel
from dj_experiment.models import Value as ValueModel
from django.core.management.base import BaseCommand, CommandError
from experiment import Case, Experiment, Field


class Command(BaseCommand):
    """Obtain an experiment configured from the catalog."""

    help = 'Obtain an experiment configured from the catalog'

    # def __init__(self):
    #     """Call the BaseCommand constructor."""
    #     super(Command, self).__init__()
    #     self.style = color_style()

    def add_arguments(self, parser):
        """Add arguments for handling the command."""
        # Named (optional) arguments start with --
        parser.add_argument('-e', '--experiment',  # argument flag
                            dest='experiment',  # argument name
                            default=None,
                            help='Create the configuration of an experiment',
                            )
        parser.add_argument('-b', '--basepath',  # argument flag
                            dest='basepath',  # argument name
                            default=None,
                            help='Create the configuration of an experiment in \
                            a specific base path',
                            )
        parser.add_argument('-x', '--export',  # argument flag
                            dest='export',  # argument name
                            default=None,
                            help='Return the configuration of an experiment in \
                            a specific base path',
                            )

    def handle(self, *args, **options):
        """Obtain an experiment configured from the catalog."""
        if not (options.get('experiment') and options.get('basepath')):
            raise CommandError(
                "configurexperiment wants at least one named argument"
            )
        else:
            experiment_name = options['experiment']
            experiment_basepath = options.get('basepath')
            if options.get('export'):
                experiment_return = options.get('export')
            else:
                experiment_return = False

        def _load_cases(self, expname):
            """."""
            caselist = list()
            try:
                casevals = [
                    exp.cases.values(
                    ) for exp in ExperimentModel.objects.filter(
                        name=expname
                    )
                ]
            except ExperimentModel.DoesNotExist:
                raise CommandError('Experiment "%s" does not exist' % expname)

            for caseval in casevals[0]:
                shortname = caseval.get("shortname")
                longname = caseval.get("longname")
                try:
                    valslist = [
                        case.casevals.values(
                        ) for case in CaseModel.objects.filter(
                            name=caseval.get("name")
                        )
                    ]
                except CaseModel.DoesNotExist:
                    raise CommandError(
                        'Case "%s" does not exist' % caseval.get("name")
                    )
                vals = [val.get("val") for val in valslist[0]]
                caselist.append(Case(shortname,
                                     longname,
                                     vals))

            return caselist

        def _load_fieldgroups(self, expname):
            """."""
            fieldgrouplist = list()
            try:
                for exp in ExperimentModel.objects.filter(name=expname):
                    fg_pk = exp.fieldgroups_id
            except ExperimentModel.DoesNotExist:
                raise CommandError('Experiment "%s" does not exist' % expname)
            try:
                fieldgroup = FieldGroupModel.objects.filter(
                    pk=fg_pk
                ).iterator()
                for fieldgroupitem in fieldgroup:
                    shortname = fieldgroupitem.shortname
                    longname = fieldgroupitem.longname
                    initialposition = fieldgroupitem.is_initialposition
                    try:
                        fieldnames = [
                            value.val for value in ValueModel.objects.filter(
                                fieldgroups=fg_pk
                            )
                        ]
                    except ValueModel.DoesNotExist:
                        raise CommandError(
                            'ValueModel with fieldgroups id "%s" \
                             does not exist' % fg_pk)
                    fieldgrouplist.append(Field(shortname,
                                                longname,
                                                fieldnames,
                                                initialposition))
            except FieldGroupModel.DoesNotExist:
                raise CommandError('FieldGroup id "%s" does not exist' % fg_pk)
            return fieldgrouplist

        def _load_timeseries(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                timeseriesval = val.get("timeseries")
            return timeseriesval

        def _load_datadir(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                datadirval = val.get("data_dir")
            return datadirval

        def _load_casepath(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                casepathval = val.get("case_path")
            return casepathval

        def _load_initialposition(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                initialpositionval = val.get("initial_position")
            return initialpositionval

        def _load_outputprefix(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                outputprefixval = val.get("output_prefix")
            return outputprefixval

        def _load_outputsuffix(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                outputsuffixval = val.get("output_suffix")
            return outputsuffixval

        def _load_validatedata(self, expname):
            """."""
            vals = ExperimentModel.objects.filter(name=expname).values()
            for val in vals.iterator():
                validatedataval = val.get("validate_data")
            return validatedataval

        _BASE_DIR = experiment_basepath
        xperiment = Experiment(
            experiment_name,
            cases=_load_cases(self, experiment_name),
            fieldgroups=_load_fieldgroups(self, experiment_name),
            timeseries=_load_timeseries(self, experiment_name),
            data_dir=os.path.join(_BASE_DIR,
                                  _load_datadir(self, experiment_name)),
            case_path=_load_casepath(self, experiment_name),
            initial_position=_load_initialposition(self, experiment_name),
            output_prefix=_load_outputprefix(self, experiment_name),
            output_suffix=_load_outputsuffix(self, experiment_name),
            validate_data=_load_validatedata(self, experiment_name)
        )

        if experiment_return:
            self.stdout.ending = ''
            self.stdout.write(xperiment.to_dict())
            logging.debug(xperiment.to_dict())
            return xperiment.to_dict()
            # self.stdout.write(self.style.NOTICE(
            #     'Successfully returned configuration \
            #     for "%s"' % experiment_name))
            logging.info('Successfully returned configuration \
                for "%s"' % experiment_name)
        self.stdout.write(self.style.NOTICE(
            'Successfully created configuration for "%s"' % experiment_name))

        logging.info('Command executed successfully!')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_experiment', '0004_auto_20170803_1330'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='experiment',
            name='name',
        ),
        migrations.AddField(
            model_name='basemodel',
            name='name',
            field=models.CharField(
                default='rkh', unique=True, max_length=100),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_experiment', '0006_auto_20170803_0851'),
    ]

    operations = [
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default=b'RCM data', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default=b'_', max_length=1),
        ),
    ]



from dj_experiment.models import Catalog
from django.core.management.base import BaseCommand, CommandError
from humanfriendly.tables import format_pretty_table


class Command(BaseCommand):
    """Retrieve information about experiments from the catalog."""

    help = 'Retrieve information about experiments from the catalog'

    def add_arguments(self, parser):
        """Add arguments for handling the command."""
        pass

    def handle(self, *args, **options):
        """Query all the entries from the catalog model."""
        if args:
            raise CommandError("infocatalog takes no arguments")
        column_names = ['Catalog Name']
        # [self.stdout.write(cat, ending='') for cat in Catalog.objects.all()]
        catalog = [cat.name for cat in Catalog.objects.all()]

        self.stdout.write(format_pretty_table(
            [catalog, ], column_names))



import os

from appconf import AppConf
from django.conf import settings


class DjExperimentAppConf(AppConf):
    DATA_DIR = "./"
    BASE_DATA_DIR = os.path.join(settings.BASE_DIR, 'data')
    SEPARATOR = "."
    OUTPUT_PREFIX = ""
    OUTPUT_SUFFIX = ".nc"
    CELERY_BROKER_URL = 'amqp://guest:guest@localhost:5672//'
    CELERY_RESULT_BACKEND = 'rpc://'
    TAGGIT_CASE_INSENSITIVE = True

    class Meta:
        prefix = 'dj_experiment'
        holder = 'dj_experiment.conf.settings'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-02 17:30
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dj_experiment', '0003_auto_20170802_1708'),
    ]

    operations = [
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-02 17:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('dj_experiment', '0002_auto_20170802_1206'), ('dj_experiment', '0003_auto_20170802_1708'), ('dj_experiment', '0004_auto_20170802_1230')]

    dependencies = [
        ('dj_experiment', '0001_initial_squashed_0006_auto_20170802_0904'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='catalog',
            name='xperiment',
        ),
        migrations.AddField(
            model_name='catalog',
            name='xperiments',
            field=models.ForeignKey(default='', on_delete=django.db.models.deletion.CASCADE, related_name='catalogs', to='dj_experiment.Experiment'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default='./', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default='.', max_length=1),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default=b'RCM data', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default=b'_', max_length=1),
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.views.generic import TemplateView

from . import views

urlpatterns = [
    url(
        regex="^experiment/~create/$",
        view=views.ExperimentCreateView.as_view(),
        name='experiment_create',
    ),
    url(
        regex="^experiment/(?P<pk>\d+)/~delete/$",
        view=views.ExperimentDeleteView.as_view(),
        name='experiment_delete',
    ),
    url(
        regex="^experiment/(?P<pk>\d+)/$",
        view=views.ExperimentDetailView.as_view(),
        name='experiment_detail',
    ),
    url(
        regex="^experiment/(?P<pk>\d+)/~update/$",
        view=views.ExperimentUpdateView.as_view(),
        name='experiment_update',
    ),
    url(
        regex="^experiment/$",
        view=views.ExperimentListView.as_view(),
        name='experiment_list',
    ),
    url(
        regex="^case/~create/$",
        view=views.CaseCreateView.as_view(),
        name='case_create',
    ),
    url(
        regex="^case/(?P<pk>\d+)/~delete/$",
        view=views.CaseDeleteView.as_view(),
        name='case_delete',
    ),
    url(
        regex="^case/(?P<pk>\d+)/$",
        view=views.CaseDetailView.as_view(),
        name='case_detail',
    ),
    url(
        regex="^case/(?P<pk>\d+)/~update/$",
        view=views.CaseUpdateView.as_view(),
        name='case_update',
    ),
    url(
        regex="^case/$",
        view=views.CaseListView.as_view(),
        name='case_list',
    ),
    url(
        regex="^fieldgroup/~create/$",
        view=views.FieldGroupCreateView.as_view(),
        name='fieldgroup_create',
    ),
    url(
        regex="^fieldgroup/(?P<pk>\d+)/~delete/$",
        view=views.FieldGroupDeleteView.as_view(),
        name='fieldgroup_delete',
    ),
    url(
        regex="^fieldgroup/(?P<pk>\d+)/$",
        view=views.FieldGroupDetailView.as_view(),
        name='fieldgroup_detail',
    ),
    url(
        regex="^fieldgroup/(?P<pk>\d+)/~update/$",
        view=views.FieldGroupUpdateView.as_view(),
        name='fieldgroup_update',
    ),
    url(
        regex="^fieldgroup/$",
        view=views.FieldGroupListView.as_view(),
        name='fieldgroup_list',
    ),
    url(
        regex="^value/~create/$",
        view=views.ValueCreateView.as_view(),
        name='value_create',
    ),
    url(
        regex="^value/(?P<pk>\d+)/~delete/$",
        view=views.ValueDeleteView.as_view(),
        name='value_delete',
    ),
    url(
        regex="^value/(?P<pk>\d+)/$",
        view=views.ValueDetailView.as_view(),
        name='value_detail',
    ),
    url(
        regex="^value/(?P<pk>\d+)/~update/$",
        view=views.ValueUpdateView.as_view(),
        name='value_update',
    ),
    url(
        regex="^value/$",
        view=views.ValueListView.as_view(),
        name='value_list',
    ),
    url(
        regex="^catalog/~create/$",
        view=views.CatalogCreateView.as_view(),
        name='catalog_create',
    ),
    url(
        regex="^catalog/(?P<pk>\d+)/~delete/$",
        view=views.CatalogDeleteView.as_view(),
        name='catalog_delete',
    ),
    url(
        regex="^catalog/(?P<pk>\d+)/$",
        view=views.CatalogDetailView.as_view(),
        name='catalog_detail',
    ),
    url(
        regex="^catalog/(?P<pk>\d+)/~update/$",
        view=views.CatalogUpdateView.as_view(),
        name='catalog_update',
    ),
    url(
        regex="^catalog/$",
        view=views.CatalogListView.as_view(),
        name='catalog_list',
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_experiment', '0002_auto_20170802_1206_squashed_0004_auto_20170802_1230'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='catalog',
            name='xperiments',
        ),
        migrations.AddField(
            model_name='catalog',
            name='xperiments',
            field=models.ManyToManyField(related_name='catalogs', verbose_name=b'Catalog experiments', to='dj_experiment.Experiment'),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default='./', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default='.', max_length=1),
        ),
    ]



# -*- coding: utf-8 -*-
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    UpdateView,
    ListView
)

from .models import (
	Experiment,
	Case,
	FieldGroup,
    Value,
	Catalog,
)


class ExperimentCreateView(CreateView):

    model = Experiment


class ExperimentDeleteView(DeleteView):

    model = Experiment


class ExperimentDetailView(DetailView):

    model = Experiment


class ExperimentUpdateView(UpdateView):

    model = Experiment


class ExperimentListView(ListView):

    model = Experiment


class CaseCreateView(CreateView):

    model = Case


class CaseDeleteView(DeleteView):

    model = Case


class CaseDetailView(DetailView):

    model = Case


class CaseUpdateView(UpdateView):

    model = Case


class CaseListView(ListView):

    model = Case


class FieldGroupCreateView(CreateView):

    model = FieldGroup


class FieldGroupDeleteView(DeleteView):

    model = FieldGroup


class FieldGroupDetailView(DetailView):

    model = FieldGroup


class FieldGroupUpdateView(UpdateView):

    model = FieldGroup


class FieldGroupListView(ListView):

    model = FieldGroup


class ValueCreateView(CreateView):

    model = Value


class ValueDeleteView(DeleteView):

    model = Value


class ValueDetailView(DetailView):

    model = Value


class ValueUpdateView(UpdateView):

    model = Value


class ValueListView(ListView):

    model = Value


class CatalogCreateView(CreateView):

    model = Catalog


class CatalogDeleteView(DeleteView):

    model = Catalog


class CatalogDetailView(DetailView):

    model = Catalog


class CatalogUpdateView(UpdateView):

    model = Catalog


class CatalogListView(ListView):

    model = Catalog



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-02 14:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    replaces = [('dj_experiment', '0001_initial'), ('dj_experiment', '0002_auto_20170802_0612'), ('dj_experiment', '0003_auto_20170802_1118'), ('dj_experiment', '0004_auto_20170802_0644'), ('dj_experiment', '0005_auto_20170802_1144'), ('dj_experiment', '0006_auto_20170802_0904')]

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BaseModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
                ('slug', django_extensions.db.fields.AutoSlugField(blank=True, editable=False, populate_from=b'title', verbose_name='slug')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
        ),
        migrations.CreateModel(
            name='Value',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('val', models.CharField(max_length=100)),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
        ),
        migrations.CreateModel(
            name='Case',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='dj_experiment.BaseModel')),
                ('shortname', models.CharField(max_length=10)),
                ('longname', models.CharField(max_length=100)),
                ('casevals', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cases', to='dj_experiment.Value')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
            bases=('dj_experiment.basemodel',),
        ),
        migrations.CreateModel(
            name='Catalog',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='dj_experiment.BaseModel')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
            bases=('dj_experiment.basemodel',),
        ),
        migrations.CreateModel(
            name='Experiment',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='dj_experiment.BaseModel')),
                ('name', models.CharField(max_length=100)),
                ('timeseries', models.BooleanField(default=False)),
                ('data_dir', models.CharField(default=b'./', max_length=250)),
                ('separator', models.CharField(default=b'.', max_length=1)),
                ('case_path', models.CharField(default=None, max_length=100)),
                ('initial_position', models.BooleanField(default=False)),
                ('output_prefix', models.CharField(default=b'', max_length=100)),
                ('output_suffix', models.CharField(default=b'.nc', max_length=100)),
                ('validate_data', models.BooleanField(default=True)),
                ('cases', models.ManyToManyField(related_name='experiments', to=b'dj_experiment.Case', verbose_name=b'Esperiment cases')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
            bases=('dj_experiment.basemodel',),
        ),
        migrations.CreateModel(
            name='FieldGroup',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='dj_experiment.BaseModel')),
                ('shortname', models.CharField(max_length=50)),
                ('longname', models.CharField(max_length=100)),
                ('is_initialposition', models.BooleanField(default=False)),
                ('fieldnames', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='fieldgroups', to='dj_experiment.Value')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'abstract': False,
                'get_latest_by': 'modified',
            },
            bases=('dj_experiment.basemodel',),
        ),
        migrations.AddField(
            model_name='experiment',
            name='fieldgroups',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='experiments', to='dj_experiment.FieldGroup'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='xperiment',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='experiments', to='dj_experiment.Experiment'),
        ),
        migrations.AlterField(
            model_name='case',
            name='shortname',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default=b'RCM data', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default=b'_', max_length=1),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_experiment', '0011_auto_20170829_0841'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataset',
            name='dsfile',
            field=models.FilePathField(path='./', max_length=1024, recursive=True),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='data_dir',
            field=models.CharField(default='./', max_length=250),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='separator',
            field=models.CharField(default='.', max_length=1),
        ),
    ]



from __future__ import absolute_import

import logging
import os
import time

from celery import shared_task
from celery.contrib import rdb
from dj_experiment.models import Case as ModCase
from dj_experiment.models import Experiment as ModExperiment
from dj_experiment.models import FieldGroup as ModFieldGroup
from dj_experiment.models import Value as ModValue
from dj_experiment.models import (CaseBelongingness, CaseKeyValue, Dataset,
                                  FieldBelongingness, FieldKeyValue)
from dj_experiment.tasks.utils.extract_files import (get_experiment, get_files,
                                                     get_keywords)


@shared_task
def longtime_add(x, y):
    print 'long time task begins'
    # sleep 5 seconds
    time.sleep(5)
    print 'long time task finished'
    return x + y


@shared_task
def netcdf_save(exp_id, dir):
    print 'netcdf save task begins'
    # do something
    xperiment = ModExperiment.objects.get(pk=exp_id)
    xpname = xperiment.name
    print 'xpname=%s' % xpname
    xpinst = get_experiment(xpname, dir)
    # rdb.set_trace()
    print xpinst
    xpfiles = get_files(xpinst, xpinst.data_dir)
    if len(xpfiles) == 0:
        logging.info(
            "No results, the number of files retrieved is %s" % len(xpfiles))
    else:
        for xpfile in xpfiles:
            logging.debug("file is the following ===> \n %s" % xpfile[0])
            logging.debug("xpfile type is %s" % type(xpfile))
            logging.info("############################################# \
PROCESSING NEW FILE \
# ")
            logging.info("Processing file =====> %s" % xpfile[0])
            xpoutfile, keywords, fieldcases = get_keywords(xpinst, xpfile)
            logging.info(
                "Retrieve keywords which are the following ===> %s" % keywords
            )

            if (fieldcases[0] and fieldcases[1]):
                # retrieve all couples key/value ids for FieldKeyValue
                # {u'temp': u'tas'}
                kvfield = tuple()
                kvfglist = list()
                fielddict = fieldcases[0]
                for key, value in fielddict.iteritems():
                    ofg = ModFieldGroup.objects.get(shortname=key)
                    kvfield += ((ofg.id, ofg.fieldnames.get(val=value).id),)
                # save record into FieldKeyValue for each couple
                for k, v in kvfield:
                    okvfg = FieldKeyValue.objects.create(
                        fieldname=ModFieldGroup.objects.get(pk=k),
                        value=ModValue.objects.get(pk=v)
                    )
                    kvfglist.append(okvfg.id)
                # retrieve all couples key/value ids for CaseKeyValue
                # {u'fixedperiod': u'19510101-21001231', etc.. }
                kvcases = tuple()
                kvclist = list()
                casesdict = fieldcases[1]
                for key, value in casesdict.iteritems():
                    oc = ModCase.objects.get(shortname=key)
                    kvcases += ((oc.id, oc.casevals.get(val=value).id),)
                # save record into CaseKeyValue for each couple
                for k, v in kvcases:
                    okvfg = CaseKeyValue.objects.create(
                        case=ModCase.objects.get(pk=k),
                        value=ModValue.objects.get(pk=v)
                    )
                    kvclist.append(okvfg.id)
                dsobj = Dataset.objects.create(title=xpoutfile[0],
                                               name=xpoutfile[0],
                                               dsfilename=xpoutfile[0],
                                               dsfile=os.path.join(
                    xpinst.data_dir,
                    xpoutfile[3],
                    xpoutfile[0]
                ))
                # save record into FieldBelongingness for each id in the list
                for obj in kvfglist:
                    ofb = FieldBelongingness(
                        dataset=dsobj,
                        keyvalue=FieldKeyValue.objects.get(pk=obj)
                    )
                    ofb.save()

                # save record into CaseBelongingness for each id in the list
                for obj in kvclist:
                    ocb = CaseBelongingness(
                        dataset=dsobj,
                        keyvalue=CaseKeyValue.objects.get(pk=obj)
                    )
                    ocb.save()
                # add keywords to dataset object
                dsobj.tags.add(*keywords)

            else:
                logging.info("Skipping file to database insertion")

    print 'netcdf save task finished'
    # return query count from db
    return Dataset.objects.count()



import ast
import logging
import os
import sys
from StringIO import StringIO

import scandir
from dj_experiment.conf import settings
from dj_experiment.management.commands.configurexperiment import \
    Command as ConfigureExperiment
from experiment import Case, Experiment, Field


def get_experiment(namedexp, rootdirexp):

    # set default root data dir if passed directory doesn't exist
    if not os.path.exists(rootdirexp):
        logging.debug("Input root directory %s doesn't exist, default \
                value %s" % (rootdirexp, settings.DJ_EXPERIMENT_BASE_DATA_DIR))
        rootdirexp = settings.DJ_EXPERIMENT_BASE_DATA_DIR

    old_stdout = sys.stdout
    # This variable will store everything that is sent to the
    # standard output
    cmd = StringIO()
    sys.stdout = cmd

    # Here we can call anything we like, like external modules,
    # and everything that they will send to standard output will be
    # stored on "cmd"
    ce = ConfigureExperiment()
    ce.handle(experiment=namedexp,
              basepath=rootdirexp,
              export=True)

    # Redirect again the std output to screen
    sys.stdout = old_stdout
    xperkwargs = ast.literal_eval(cmd.getvalue())
    logging.debug("xperkwargs======>%s" % xperkwargs)

    # Try to instantiate cases
    logging.debug("Reading cases")
    try:
        caseitems = xperkwargs['cases'].items()
    except TypeError as e:
        raise ValueError(
            "Couldn't retrieve `caseitems` \
            because `xperkwargs` ===> %s" % xperkwargs)
        logging.debug("Exception is %s" % e)
    cases = []
    for case_short, case_kws in caseitems:
        logging.debug("      {}: {}".format(case_short, case_kws))
        cases.append(Case(case_short, **case_kws))
    xperkwargs['cases'] = cases

    # Try to instantiate fieldgroups
    logging.debug("Reading fieldgroup")
    fieldgroups = []
    for fieldgroup_short, fieldgroup_kws in xperkwargs['fieldgroups'].items():
        logging.debug("      {}: {}".format(fieldgroup_short, fieldgroup_kws))
        fieldgroups.append(Field(fieldgroup_short, **fieldgroup_kws))
    xperkwargs['fieldgroups'] = fieldgroups

    # Create and return the Experiment
    xperiment = Experiment(**xperkwargs)
    logging.debug(xperiment.to_dict())
    return xperiment


def get_files(inst, datadir):

    extension = inst.output_suffix
    return _build_fullfilepathname(extension, datadir)


def get_keywords(inst, filetuple):

    # catch year as extra_keywords and clean it
    searchfile = filetuple[0]
    logging.debug("Search keywords for file %s" % searchfile)
    fields = inst.get_fieldgroup_vals(inst.fieldgroups[0])[0]
    cleanfile, extra_keywords = _build_extrakeywords(searchfile,
                                                     inst.all_case_vals(),
                                                     fields,
                                                     inst.output_suffix)
    logging.debug("Cleaned file ===> %s" % cleanfile)
    searchcleanedfile = os.path.join(settings.DJ_EXPERIMENT_DATA_DIR,
                                     str(filetuple[3]),
                                     str(cleanfile))
    logging.debug("Searching for matching cases \
                  with this file ===> %s" % searchcleanedfile)
    filetuple += (searchcleanedfile, )
    fieldcaselist = inst.get_cases_fromfile(searchcleanedfile)
    try:
        matchdict = [
            ({inst.fieldgroups[0]:field},
             matchcases[0]
             ) for field, matchcases in fieldcaselist if matchcases
        ]
        logging.debug(
            "Key/value dictionary of matched cases is ===> %s" % matchdict
        )
        case_keywords = matchdict[0][1].values()
        logging.debug("Case keywords from file %s" % case_keywords)
        logging.debug("Extra keywords from file %s" % extra_keywords)
        keywords = case_keywords + extra_keywords
        if keywords:
            keywords = list(set(keywords))
            logging.info(
                "Final return \nKEYWORDS===>%s\nfrom\nFILE===>%s " % (
                    keywords, filetuple[0])
            )
            return filetuple, keywords, matchdict[0]
    except IndexError as e:
        logging.error(
            "Passed file %s doesn't match any case" % searchcleanedfile
        )
        logging.debug("Exception is %s" % e)
        logging.error(
            "Not able to retrieve any keyword for file %s" % filetuple[0]
        )
        return filetuple, {}, ({}, {})


def _build_fullfilepathname(ext, root):

    ext = ext.lower()
    tpl = tuple()

    for dirname, dirs, files in scandir.walk(root):
        for file_ in files:
            if file_.lower().endswith(ext):
                logging.debug("file_=%s" % file_)
                logging.debug("dirname=%s" % dirname)
                logging.debug("fullname=%s" % os.path.join(dirname, file_))
                logging.debug("relpath=%s" % os.path.relpath(dirname, root))
                tpl += ((file_,
                         dirname,
                         os.path.join(dirname, file_),
                         os.path.relpath(dirname, root)), )
    return tpl


def _build_extrakeywords(fn, casevals, fieldvals, filesuffix):

    foundwords = list()
    extrawords = list()

    # find words not present along cases
    words = os.path.splitext(str(fn))[0].split(
        settings.DJ_EXPERIMENT_SEPARATOR
    )
    logging.debug("Input words are ===> %s" % words)
    for word in words:
        if any(
            word in s for s in [
                val for vals in casevals for val in vals
            ]
        ):
            foundwords.append(word)
            logging.debug("Foundwords are ===> %s" % foundwords)
    extrawords = [item for item in words if item not in foundwords]
    logging.debug("Extrawords are ===> %s" % extrawords)
    cleanextrawords = [
        cleanextra for cleanextra in extrawords if cleanextra not in fieldvals
    ]
    logging.debug("Cleanextrawords are ===> %s" % cleanextrawords)
    cleanwords = [clean for clean in words if clean not in cleanextrawords]
    cleanwords.append(filesuffix)
    logging.debug("Cleanwords are ===> %s" % cleanwords)
    fn = settings.DJ_EXPERIMENT_SEPARATOR.join(cleanwords)
    logging.debug("Clean content of fn is ===> %s" % fn)
    extrawords = map(unicode, extrawords)
    return fn, extrawords



# -*- coding: utf-8 -*-

import os

from dj_experiment.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import (TimeStampedModel,
                                         TitleSlugDescriptionModel)
from taggit.managers import TaggableManager


class BaseModel(TimeStampedModel, TitleSlugDescriptionModel):
    """
    Abstract model to store information about an experiment.

    `title` `description`, and `slug` fields inherited from django-extensions
    TitleSlugDescriptionModel.

    Keyword arguments:
    title - string
    description - string
    slug - string
    """

    name = models.CharField(max_length=255, unique=True)

    def __str__(self):
        """Do a text representation of all models."""
        return "{name}".format(name=self.name)


class Value(TimeStampedModel):
    """Represent values of all cases and fieldgroups."""

    val = models.CharField(blank=False, max_length=100, unique=True)

    def __str__(self):
        """Do a text representation of the model."""
        return "{val}".format(val=self.val)


class Case(BaseModel):
    """Represent a single case of an experiment."""

    shortname = models.CharField(blank=False, max_length=50)
    longname = models.CharField(blank=False, max_length=100)
    casevals = models.ManyToManyField(
        Value, related_name='cases', verbose_name=_('Case values'))


class FieldGroup(BaseModel):
    """Represent a single group of fields."""

    shortname = models.CharField(blank=False, max_length=50)
    longname = models.CharField(blank=False, max_length=100)
    fieldnames = models.ManyToManyField(
        Value, related_name='fieldgroups', verbose_name=_('FieldGroup values'))
    is_initialposition = models.BooleanField(default=False)


class Experiment(BaseModel):
    """Represent an experiment."""

    cases = models.ManyToManyField(
        Case, related_name='experiments', verbose_name=_('Esperiment cases'))
    fieldgroups = models.ForeignKey(FieldGroup, related_name='experiments')
    timeseries = models.BooleanField(default=False)
    data_dir = models.CharField(
        default=settings.DJ_EXPERIMENT_DATA_DIR, max_length=250)
    separator = models.CharField(
        default=settings.DJ_EXPERIMENT_SEPARATOR, max_length=1)
    case_path = models.CharField(default=None, max_length=100)
    initial_position = models.BooleanField(default=False)
    output_prefix = models.CharField(
        default=settings.DJ_EXPERIMENT_OUTPUT_PREFIX, max_length=100)
    output_suffix = models.CharField(
        default=settings.DJ_EXPERIMENT_OUTPUT_SUFFIX, max_length=100)
    validate_data = models.BooleanField(default=True)


class Catalog(BaseModel):
    """Represent a catalog of all experiments."""

    xperiments = models.ManyToManyField(
        Experiment,
        related_name='catalogs',
        verbose_name=_('Catalog experiments'))


class CaseKeyValue(models.Model):
    """Represent a key/value instance of a Case experiment."""

    case = models.ForeignKey(Case)
    value = models.ForeignKey(Value)


class FieldKeyValue(models.Model):
    """Represent a key/value instance of a Field experiment."""

    fieldname = models.ForeignKey(FieldGroup)
    value = models.ForeignKey(Value)


class Dataset(BaseModel):
    """Represent an output dataset for an experiment."""

    dsfilename = models.CharField(
        max_length=256, db_index=True, unique=True)
    dsfile = models.FilePathField(path=os.path.join(
        settings.DJ_EXPERIMENT_BASE_DATA_DIR,
        settings.DJ_EXPERIMENT_DATA_DIR
    ),
        recursive=True,
        max_length=1024)
    tags = TaggableManager()
    casekeyvalues = models.ManyToManyField(
        CaseKeyValue,
        through='CaseBelongingness',
        through_fields=('dataset', 'keyvalue'))
    fieldkeyvalues = models.ManyToManyField(
        FieldKeyValue,
        through='FieldBelongingness',
        through_fields=('dataset', 'keyvalue'))

    def __str__(self):
        return self.dsfilename


class CaseBelongingness(models.Model):
    """Represent bridge to a key,value case for a dataset of an experiment."""

    dataset = models.ForeignKey(Dataset)
    keyvalue = models.ForeignKey(CaseKeyValue)


class FieldBelongingness(models.Model):
    """Represent bridge to a key,value field for a dataset of an experiment."""

    dataset = models.ForeignKey(Dataset)
    keyvalue = models.ForeignKey(FieldKeyValue)



from django.contrib import admin

from .models import (Case, CaseBelongingness, CaseKeyValue, Catalog, Dataset,
                     Experiment, FieldBelongingness, FieldGroup, FieldKeyValue,
                     Value)

admin.site.register(Experiment)
admin.site.register(Case)
admin.site.register(FieldGroup)
admin.site.register(Catalog)
admin.site.register(Value)
admin.site.register(CaseKeyValue)
admin.site.register(FieldKeyValue)
admin.site.register(Dataset)
admin.site.register(CaseBelongingness)
admin.site.register(FieldBelongingness)
